/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <getopt.h>
#include <unistd.h>

#include "aubio.h"	

#include "utils.h"

const char * output_filename = NULL;
const char * input_filename = NULL;

int verbose = 0;
int usejack = 0;

int main (int argc, char **argv) {  
	/* settings */
	int buffer_size =  512, overlap_size = 128, dsample = 1,  usample = 2;
 	onsetdetection_t odtype = phase;

	int channels = 2, frames;

	/* file objects */
	aubio_file_t * file;
	aubio_file_t * fileout;

	/* phase vocoder */
	aubio_pvoc_t * pv;
	
	/* onset detectors */
	onsetdetector_t *o;
	fvec_t *onset;
	fvec_t *ibuf, *ibuf2;
	fvec_t *obuf, *obuf2;
	cvec_t *fftgrain;
	
	int rwsize = overlap_size*dsample;
	// downsample factor 8
	//aubio_resampler_t *downsample = new_aubio_resampler(1.0/(float)dsample,1);
	// upsample factor 2
	aubio_resampler_t *upsample = new_aubio_resampler((float)usample,1);
	// parsing command line argument
	parse_args(argc, argv);

	debug("Opening  files ...\n");
	file = new_file_ro (input_filename);
	file_info(file);
	channels = file_channels(file);
	fileout = new_file_wo(file, output_filename);
		
	debug("Phase voc init ... \n");
	pv = new_aubio_pvoc(buffer_size, overlap_size, channels);
	
	ibuf = new_fvec(rwsize, channels);
	obuf = new_fvec(rwsize, channels);
	ibuf2 = new_fvec(overlap_size, channels);
	obuf2 = new_fvec(overlap_size, channels);
	fftgrain = new_cvec(buffer_size, channels);
	
	/* onset init */
	o = new_onsetdetection(odtype, buffer_size,channels);
	onset = new_fvec(1, channels);

	/* reading and processing data from file */
	frames = 0;
	while ((rwsize == file_read(file, rwsize, ibuf) )) {
		//aubio_resampler_process(downsample,ibuf,ibuf2);
		aubio_pvoc_do(pv, ibuf, fftgrain);
		onsetdetection(o, fftgrain, onset);
		//errmsg("%f\n",onset->data[0][0]);
		errmsg("%'.30f\n",onset->data[0][0]);

		aubio_pvoc_rdo(pv, fftgrain, obuf2);
		aubio_resampler_process(upsample,obuf2,obuf);
		file_write(fileout,rwsize,obuf);
		frames++;
	}
	debug("Processed %d frames of %d samples (%d frames).\n", 
			frames, buffer_size, frames*overlap_size);
	debug("The file was %d samples.\n", frames*overlap_size*dsample);

	del_file(file);
	del_file(fileout);

	del_aubio_pvoc(pv);

	debug("End of program.\n");
	fflush(stderr);

	return 0;
}

