/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 */

#include "aubio_priv.h"
#include "scale.h"

struct _scale_t {
	float ilow;
	float ihig;
	float olow;
	float ohig;

	float scaler;
	float irange;
	
	/* not implemented yet : type in/out data
	bool inint;
	bool outint;
	*/
};


/********************************************************
 * Object Memory Allocation
 */

static scale_t * scale_malloc(void);
static void scale_free(scale_t * s);

scale_t * scale_malloc() {
	scale_t * s = AUBIO_NEW(scale_t);
	return s;
}

void scale_free(scale_t * s) {
	AUBIO_FREE(s);
}

/***
 * Object creation/deletion calls
 */
scale_t * scale_new (float ilow, float ihig, float olow, float ohig	){
	scale_t * s = scale_malloc();
	scale_set (s, ilow, ihig, olow, ohig);
	return s;	
}

void scale_delete(scale_t *s) {
	scale_free(s);
}

/***
 * set parmams
 */
void scale_set (scale_t *s, float ilow, float ihig, float olow, float ohig) 
{
	float inputrange = ihig - ilow;
	float outputrange= ohig - olow;
	s->ilow = ilow;
	s->ihig = ihig;
	s->olow = olow;
	s->ohig = ohig;
	if (inputrange == 0 )
		s->scaler = 0.0f;
	else {
		s->scaler = outputrange/inputrange;
		if (inputrange < 0 )
			inputrange = inputrange * -1.0f;
	}
}

/***
 * do it
 */
void scale_do (scale_t *s, fvec_t *input) 
{
	unsigned int i, j;
	for (i=0; i < input->channels; i++){
		for (j=0;  j < input->length; j++){
			input->data[i][j] -= s->ilow;
			input->data[i][j] *= s->scaler;
			input->data[i][j] += s->olow;
		}
	}
}

